@@ -1,27 +1,19 @@
 import justpy as jp
 from spreadsheet.googlesheet import GoogleSheet
-
 from lodstorage.lod import LOD
 from lodstorage.sparql import SPARQL
 from markupsafe import Markup
 import copy
 import datetime
 import re
-import os
 import pprint
 import sys
-import traceback
-
-from jp.widgets import LodGrid,MenuButton, MenuLink, QAlert,QPasswordDialog
-
-from spreadsheet.version import Version
+import onlinespreadsheet.version as version
+from jpwidgets.widgets import LodGrid, QPasswordDialog
+from jpwidgets.bt5widgets import App, Alert, IconButton, Switch
 from spreadsheet.wikidata import Wikidata
 from spreadsheet.wbquery import WikibaseQuery
 
-DEBUG = 0
-from argparse import ArgumentParser
-from argparse import RawDescriptionHelpFormatter
-
 class WikidataGrid():
     '''
     the tabular data to work with
@@ -226,11 +218,12 @@ class GridSync():
                         if column in lodRow:
                             self.checkCell(viewLodRow,column,value,propVarname,propType,propLabel,propUrl)   
 
-class GoogleSheetWikidataImport():
+class GoogleSheetWikidataImport(App):
     '''
     reactive google sheet display to be used for wikidata import of the content
     '''
-    def __init__(self,url,sheetNames:list,pk:str,endpoint:str,lang:str="en",debug:bool=False):
+    
+    def __init__(self,version):
         '''
         constructor
         
@@ -242,38 +235,21 @@ class GoogleSheetWikidataImport():
             lang(str): the languate to use for labels
             debug(bool): if True show debug information
         '''
-        self.debug=debug
-        self.url=url
-        self.sheetNames=sheetNames
-        self.sheetName=sheetNames[0]
-        self.pk=pk
-        self.endpoint=endpoint
-        self.sparql=SPARQL(self.endpoint)
-        self.lang=lang
+        App.__init__(self, version,title="Google Spreadsheet Wikidata Import")
+        # url,sheetNames:list,pk:str,endpoint:str,lang:str="en",debug:bool=False
         # @TODO make configurable
+        self.metaDataSheetName="WikidataMetadata"
         self.wd=Wikidata("https://www.wikidata.org",debug=True)
         self.agGrid=None
         self.wdgrid=None
         self.dryRun=True
-        
-    def handleException(self,ex):
-        '''
-        handle the given exception
-        
-        Args:
-            ex(Exception): the exception to handle
-        '''
-        errorMsg=str(ex)
-        trace=""
-        if self.debug:
-            trace=traceback.format_exc()
-        errorMsgHtml=f"{errorMsg}<pre>{trace}</pre>"
-        self.errors.inner_html=errorMsgHtml
-        print(errorMsg)
-        if self.debug:
-            print(trace)
+        self.ignoreErrors=False
+        self.addMenuLink(text='Home',icon='home', href="/")
+        self.addMenuLink(text="docs",icon="file-document",href='https://wiki.bitplan.com/index.php/PyOnlineSpreadSheetEditing')
+        self.addMenuLink(text='github',icon='github', href="https://github.com/WolfgangFahl/pyOnlineSpreadSheetEditing")
+    
             
-    def load(self,url:str,sheetName:str):
+    def load(self,url:str,sheetName:str,metaDataSheetName="WikidataMetadata"):
         '''
         load my googlesheet, wikibaseQueries and dataframe
         
@@ -281,7 +257,7 @@ class GoogleSheetWikidataImport():
             url(str): the url to load the spreadsheet from
             sheetName(str): the sheetName of the sheet/tab to load
         '''
-        wbQueries=WikibaseQuery.ofGoogleSheet(url, "Wikidata", debug=self.debug)
+        wbQueries=WikibaseQuery.ofGoogleSheet(url, metaDataSheetName, debug=self.debug)
         self.wdgrid=WikidataGrid(wbQueries)
         self.gs=GoogleSheet(url)    
         self.gs.open([sheetName])  
@@ -297,6 +273,7 @@ class GoogleSheetWikidataImport():
         if self.debug:
             print(msg)
         try:
+            self.clearErrors()
             # prepare syncing the table results with the wikibase query result
             gridSync=GridSync(self.wdgrid,self.sheetName,self.pk,debug=self.debug)
             # query based on table content
@@ -331,14 +308,16 @@ class GoogleSheetWikidataImport():
         # set html columns according to types that have links
         self.agGrid.html_columns = self.wdgrid.getHtmlColums(self.sheetName)
          
-    def reload(self,_msg=None):
+    def reload(self,_msg=None,clearErrors=True):
         '''
         reload the table content from myl url and sheet name
         '''
-        self.load(self.url,self.sheetName)
+        if clearErrors:
+            self.clearErrors()
+        self.load(self.url,self.sheetName,self.metaDataSheetName)
         # is there already agrid?
         if self.agGrid is None:
-            self.agGrid = LodGrid(a=self.container)
+            self.agGrid = LodGrid(a=self.rowC)
         viewLod=self.wdgrid.viewLod
         self.wdgrid.linkWikidataItems(viewLod)
         self.reloadAgGrid(viewLod)
@@ -358,6 +337,7 @@ class GoogleSheetWikidataImport():
         Args:
             msg(dict): the justpy message
         '''
+       
         if self.debug:
             print(msg)
         self.sheetName=msg.value
@@ -406,10 +386,19 @@ class GoogleSheetWikidataImport():
             msg(dict): the justpy message
         '''
         self.dryRun=msg.value
+        
+    def onChangeIgnoreErrors(self,msg:dict):
+        '''
+        handle change of IgnoreErrors setting
+        
+        Args:
+            msg(dict): the justpy message
+        '''
+        self.ignoreErrors=msg.value    
             
     def loginUser(self,user):
-        self.loginButton.text=f"logout {user}"
-        self.loginButton.icon="chevron_left"
+        self.loginButton.text=f"{user}"
+        self.loginButton.iconName="logout"
         self.dryRunButton.disable=False
         
     def onloginViaDialog(self,_msg):
@@ -431,6 +420,7 @@ class GoogleSheetWikidataImport():
         if self.debug:
             print(msg)
         try:    
+            self.clearErrors()
             if self.wd.user is None:
                 self.wd.loginWithCredentials()
                 if self.wd.user is None:
@@ -443,7 +433,7 @@ class GoogleSheetWikidataImport():
                 self.dryRunButton.value=True
                 self.dryRunButton.disable=True
                 self.loginButton.text="login"
-                self.loginButton.icon="chevron_right"
+                self.loginButton.iconName   ="chevron_right"
         except Exception as ex:
                 self.handleException(ex)
      
@@ -456,13 +446,17 @@ class GoogleSheetWikidataImport():
         '''
         if self.debug:
             print(msg)
+        self.clearErrors()
         if msg.selected:
             self.rowSelected = msg.rowIndex
             write=not self.dryRun
             label=msg.data["label"]
             try:
                 mapDict=self.wdgrid.wbQueries[self.sheetName].propertiesById
-                qid,errors=self.wd.addDict(msg.data, mapDict,write=write)
+                rowData=msg.data
+                # remove index
+                rowData.pop("lodRowIndex")
+                qid,errors=self.wd.addDict(msg.data, mapDict,write=write,ignoreErrors=self.ignoreErrors)
                 if qid is not None:
                     # set item link
                     link=self.wdgrid.createLink(f"https://www.wikidata.org/wiki/{qid}", f"{label}")
@@ -475,38 +469,54 @@ class GoogleSheetWikidataImport():
                 if self.dryRun:
                     prettyData=pprint.pformat(msg.data)
                     html=Markup(f"<pre>{prettyData}</pre>")
-                    self.alertDialog.alertContent.inner_html=html
-                    self.alertDialog.alertTitle.text=f"Dry Run for {label}"
-                    self.alertDialog.value=True
+                    alert=Alert(text=html)
                     
             except Exception as ex:
                 self.handleException(ex)
 
-    def gridForDataFrame(self):
+    def setup(self):
+        self.url=self.args.url
+        self.sheetNames=self.args.sheets
+        self.sheetName=self.sheetNames[0]
+        self.pk=self.args.pk
+        self.endpoint=self.args.endpoint
+        self.sparql=SPARQL(self.endpoint)
+        self.lang="en" # @TODO Make configurable self.args.lang
+        
+    async def content(self):
         '''
         show aggrid for the given data frame
         '''
-        self.wp = jp.QuasarPage()
-        self.container=jp.Div(a=self.wp)
-        self.header=jp.Div(a=self.container)
-        self.toolbar=jp.QToolbar(a=self.header)
+        self.setup()
+        # select endpoint
+        head="""<link rel="stylesheet" href="/static/css/md_style_indigo.css">
+<link rel="stylesheet" href="/static/css/pygments.css">
+"""
+        # extend the justpy Webpage with the given head parameters
+        self.wp=self.getWp(head)
+        self.rowA=jp.Div(classes="row",a=self.contentbox)
+        self.rowB=jp.Div(classes="row",a=self.contentbox)
+        self.rowC=jp.Div(classes="row",a=self.contentbox)
+        self.rowD=jp.Div(classes="row",a=self.contentbox)
+        self.errors=jp.Span(a=self.rowD,style='color:red')
+        self.header=jp.Div(a=self.rowA)
+        self.toolbar=jp.QToolbar(a=self.rowB)
         # for icons see  https://quasar.dev/vue-components/icon
         # see justpy/templates/local/materialdesignicons/iconfont/codepoints for available icons    
-        self.reloadButton=MenuButton(a=self.toolbar,text='reload',icon="refresh",click=self.reload)
-        self.checkButton=MenuButton(a=self.toolbar,text='check',icon='check_box',click=self.onCheckWikidata)
-        MenuLink(a=self.toolbar,text="docs",icon="description",href='https://wiki.bitplan.com/index.php/PyOnlineSpreadSheetEditing')
-        MenuLink(a=self.toolbar,text='github',icon='forum', href="https://github.com/WolfgangFahl/pyOnlineSpreadSheetEditing")
-        self.loginButton=MenuButton(a=self.toolbar,icon='chevron_right',text="login",click=self.onLogin)
+        self.reloadButton=IconButton(a=self.toolbar,text='',iconName="refresh-circle",click=self.reload,classes="btn btn-primary btn-sm col-1")
+        self.checkButton=IconButton(a=self.toolbar,text='',iconName='check',click=self.onCheckWikidata,classes="btn btn-primary btn-sm col-1")
+        self.loginButton=IconButton(a=self.toolbar,iconName='login',text="",click=self.onLogin,classes="btn btn-primary btn-sm col-1")
         self.passwordDialog=QPasswordDialog(a=self.wp)
-        self.alertDialog=QAlert(a=self.wp)
         #jp.Br(a=self.header)
         # url
         urlLabelText="Google Spreadsheet Url"
         self.gsheetUrl=jp.A(a=self.header,href=self.url,target="_blank",title=urlLabelText)
         self.linkIcon=jp.QIcon(a=self.gsheetUrl,name="link",size="md")
         self.urlInput=jp.Input(a=self.header,placeholder=urlLabelText,size=80,value=self.url,change=self.onChangeUrl)
-        self.dryRunButton=jp.QToggle(a=self.header,text="dry run",value=True,disable=True)
+        self.dryRunButton=Switch(a=self.header,labelText="dry run",checked=True,disable=True)
         self.dryRunButton.on("input",self.onChangeDryRun)
+        self.ignoreErrorsButton=Switch(a=self.header,labelText="ignore errors",checked=self.ignoreErrors)
+        self.ignoreErrorsButton.on("input",self.onChangeIgnoreErrors)
         jp.Br(a=self.header)
         # link to the wikidata item currently imported
         selectorClasses='w-32 m-4 p-2 bg-white'
@@ -519,72 +529,27 @@ class GoogleSheetWikidataImport():
         self.pkSelect=jp.Select(classes=selectorClasses,a=self.header,value=self.pk,
             change=self.onChangePk)
         jp.Br(a=self.header)
-        self.errors=jp.Span(a=self.container,style='color:red')
         try:
             self.reload()
         except Exception as ex:
             self.handleException(ex)
         return self.wp
-    
-    def start(self):
-        '''
-        start the reactive justpy webserver
-        '''
-        jp.justpy(self.gridForDataFrame)
-
-def main(argv=None): # IGNORE:C0111
-    '''main program.'''
-
-    if argv is None:
-        argv=sys.argv[1:]
         
-    program_name = os.path.basename(__file__)
-    program_version = "v%s" % Version.version
-    program_build_date = str(Version.updated)
-    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
-    program_shortdesc = "Wikidata Import from google spreadsheet"
-    user_name="Wolfgang Fahl"
-    program_license = '''%s
-
-  Created by %s on %s.
-  Copyright 2022 contributors. All rights reserved.
-
-  Licensed under the Apache License 2.0
-  http://www.apache.org/licenses/LICENSE-2.0
-
-  Distributed on an "AS IS" basis without warranties
-  or conditions of any kind, either express or implied.
-
-USAGE
-''' % (program_shortdesc,user_name, str(Version.date))
-
-    try:
-        # Setup argument parser
-        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
-        parser.add_argument("-d", "--debug", dest="debug",   action="store_true", help="set debug [default: %(default)s]")
-        parser.add_argument('-V', '--version', action='version', version=program_version_message)
+    def getParser(self):
+        '''
+        get the argument parser
+        '''
+        parser=App.getParser(self)
         parser.add_argument('--endpoint',help="the endpoint to use [default: %(default)s]",default="https://query.wikidata.org/sparql")
         #parser.add_argument('--dryrun', action="store_true", dest='dryrun', help="dry run only")
         parser.add_argument('--url')
         parser.add_argument('--sheets',nargs="+",required=True)
         parser.add_argument('--pk')
-        args = parser.parse_args(argv)
-        gswdi=GoogleSheetWikidataImport(args.url,args.sheets,pk=args.pk,endpoint=args.endpoint,debug=args.debug)
-        gswdi.start()
+        return parser
      
-    except KeyboardInterrupt:
-        ### handle keyboard interrupt ###
-        return 1
-    except Exception as e:
-        if DEBUG:
-            raise(e)
-        indent = len(program_name) * " "
-        sys.stderr.write(program_name + ": " + repr(e) + "\n")
-        sys.stderr.write(indent + "  for help use --help")
-        print(traceback.format_exc())
-        return 2     
-
+DEBUG = 1
 if __name__ == "__main__":
     if DEBUG:
         sys.argv.append("-d")
-    sys.exit(main())
\ No newline at end of file
+    gsimport=GoogleSheetWikidataImport(version.Version)
+    sys.exit(gsimport.mainInstance())
\ No newline at end of file
